;-------------------------------------------------------------------------
; MSP430-449STK starterkit for MSP430F449 microcontroller
; Hardware: PCB REV.A
;-------------------------------------------------------------------------
#include  "include\msp430x44x.h"
;-------------------------------------------
; PORT definitions
;-------------------------------------------

#define     B1          BIT2,&P5IN          /* Button 1        P5.2     */
#define     B2          BIT3,&P5IN          /* Button 2        P5.3     */
#define     B3          BIT4,&P5IN          /* Button 3        P5.4     */
#define     B4          BIT5,&P5IN          /* Button 4        P5.5     */
#define     LED         BIT7,&P5OUT         /* LED STATUS      P5.7     */

#define     CR        0Dh                   /* ASCII CR                   */
#define     LF        0Ah                   /* ASCII LF                   */
#define     F1_0      010h
#define     F1_1      008h
#define     F2_0      020h
#define     F2_1      010h
#define     F3_0      030h
#define     F3_1      018h
#define     F4_0      040h
#define     F4_1      020h

#include "include\lcd_const.h"

;-------------------------------------------
; Register variables
;-------------------------------------------
#define     LCD1      R8
#define     LCD2      R9
#define     FREQ0     R10
#define     FREQ1     R11
#define     INDEX     R12                   /* common use register          */
#define     SPTR      R13                   /* String RS232 send ptr        */
#define     HOURMIN   R14                   /* Hours : minutes              */
#define     SECONDS   R15                   /* Seconds RTC                  */
 
;-------------------------------------------
; Data variables
;-------------------------------------------
            org     0200h                   ;RAM address

RXTXData          dw      1                 ;RS232 working word
DataConv          db      1                 ;HEX to ASCII converter variable
LCD_update_flag   db      1                 ;update LCD flag
Blink_flag        db      1


#define     LCD_a   0000000000000001b
#define     LCD_b   0000000000010000b
#define     LCD_c   0000000100000000b
#define     LCD_d   0001000000000000b

#define     LCD_e   0000000000000001b
#define     LCD_f   0000000000010000b
#define     LCD_g   0000000100000000b
#define     LCD_dp  0001000000000000b

;-------------------------------------------
            org     01000h                  ;Code memory
;-------------------------------------------
LCD1_TBL    dw      LCD_a+LCD_b+LCD_c+LCD_d ;0
            dw      LCD_b+LCD_c             ;1
            dw      LCD_a+LCD_b+LCD_d       ;2
            dw      LCD_a+LCD_b+LCD_c+LCD_d ;3
            dw      LCD_b+LCD_c             ;4
            dw      LCD_a+LCD_c+LCD_d       ;5
            dw      LCD_a+LCD_c+LCD_d       ;6
            dw      LCD_a+LCD_b+LCD_c       ;7
            dw      LCD_a+LCD_b+LCD_c+LCD_d ;8
            dw      LCD_a+LCD_b+LCD_c+LCD_d ;9
            dw      LCD_a+LCD_b+LCD_c       ;A
            dw      LCD_c+LCD_d             ;B
            dw      LCD_a+LCD_d             ;C
            dw      LCD_b+LCD_c+LCD_d       ;D
            dw      LCD_a+LCD_d             ;E
            dw      LCD_a                   ;F
            
            
LCD2_TBL    dw      LCD_e+LCD_f             ;0
            dw      0                       ;1
            dw      LCD_e+LCD_g             ;2
            dw      LCD_g                   ;3
            dw      LCD_f+LCD_g             ;4
            dw      LCD_f+LCD_g             ;5
            dw      LCD_f+LCD_g+LCD_e       ;6
            dw      0                       ;7
            dw      LCD_e+LCD_f+LCD_g       ;8
            dw      LCD_f+LCD_g             ;9
            dw      LCD_e+LCD_f+LCD_g       ;A
            dw      LCD_e+LCD_f+LCD_g       ;B
            dw      LCD_e+LCD_f             ;C
            dw      LCD_e+LCD_g             ;D
            dw      LCD_e+LCD_f+LCD_g       ;E
            dw      LCD_e+LCD_f+LCD_g       ;F

#include  "include\449uart.h"
#include  "include\rtc.h"                               

LCD_update  push    INDEX                   ;INDEX = digit    (2 bytes per digit)
            add.w   INDEX,INDEX             ;LCD2 = position  (4 bytes per digit)
            add.w   LCD2,LCD2               ;LCD1 - temp variable            
            add.w   LCD2,LCD2               ;
            mov.w   LCD1_TBL(INDEX),LCD1    ;
            mov.b   LCD1,LCDMEM(LCD2)       ;
            swpb    LCD1                    ;
            inc.w   LCD2                    ;
            mov.b   LCD1,LCDMEM(LCD2)       ;
            mov.w   LCD2_TBL(INDEX),LCD1    ;
            inc.w   LCD2                    ;
            mov.b   LCD1,LCDMEM(LCD2)       ;
            swpb    LCD1                    ;
            inc.w   LCD2                    ;
            mov.b   LCD1,LCDMEM(LCD2)       ;
            pop     INDEX                   ;
            ret                             ;

delay       mov.w   #0,INDEX                ;
delay0      dec.w   INDEX                   ;
            jnz     delay0                  ;
            ret                             ;
LCD_clear            
            mov.w   #0,INDEX                ;
fillnxt     mov.b   #0,LCDMEM(INDEX)        ;011h
            inc.w   INDEX                   ;
            cmp.w   #20,INDEX               ;
            jnz     fillnxt                 ;
            mov.b   #0h,09Fh                ;001h
            ret                             ;
            
LCD_RTC_update
            bic.b   #WDTIE,&IE1             ;disable WDT interrupts
            mov.w   HOURMIN,INDEX           ;
            and.w   #000Fh,INDEX            ;
            mov.w   #0,LCD2                 ;
            call    #LCD_update             ;
            mov.w   HOURMIN,INDEX           ;
            rra.w   INDEX                   ;
            rra.w   INDEX                   ;
            rra.w   INDEX                   ;
            rra.w   INDEX                   ;
            and.w   #000Fh,INDEX            ;
            mov.w   #1,LCD2                 ;
            call    #LCD_update             ;
            mov.w   HOURMIN,INDEX           ;
            swpb    INDEX                   ;
            and.w   #000Fh,INDEX            ;
            mov.w   #2,LCD2                 ;
            call    #LCD_update             ;
            mov.w   #0ch,INDEX              ;'1' index in LCDMEM
            bit.w   #1000h,HOURMIN          ;
            jnz     seth1                   ;
reseth1     bic.b   #01h,LCDMEM(INDEX)      ;            
            jmp     lcdext                  ;
seth1       bis.b   #01h,LCDMEM(INDEX)      ;            
lcdext      mov.w   #0fh,INDEX              ;
            tst.b   Blink_flag              ;
            jnz     setc1                   ;
resetc1     bic.b   #01h,LCDMEM(INDEX)      ;            
            jmp     lcdext1                 ;
setc1       bis.b   #01h,LCDMEM(INDEX)      ;            
lcdext1
            bis.b   #WDTIE,&IE1             ;disable WDT interrupts
            ret                             ;
;--------------------------------------------
RESET       mov.w   #0300h,SP               ;Initialize stackpointer
SetupWDT    mov.w   #WDT_ADLY_1000,&WDTCTL  ;init WDT for 1000ms
SetupTA     mov.w   #TASSEL0+TACLR,&TACTL   ;Clear TAR & set TAR source = ACLK
            mov.b   #XCAP14PF,&FLL_CTL0     ;14 pf
            mov.b   #LCDON+LCDSTATIC+LCDP0+LCDP2,&LCDCTL ;
            mov.b   #BTFRFQ1,&BTCTL         ;

            call    #LCD_clear              ;
            call    #delay                  ;
            
            mov.b   #005h,&P1SEL            ;buzzer output
            mov.b   #005h,&P1DIR            ;
            bis.b   #080h,&P5DIR            ;LED output
            bis.b   #LED                    ;
            bic.w   #MC0+MC1,&TACTL         ;Stop Timer_A

            clr.b   Blink_flag              ;
            
            call    #UART1_setup            ;
            mov.w   #HelloS,SPTR            ;B2 string
            call    #TX_String              ;

            mov.w   #0100h,HOURMIN          ;
            mov.w   #0,SECONDS              ;
            bis.b   #WDTIE,&IE1             ;enable WDT interrupts
            eint                            ;
mainLCD
            call    #LCD_RTC_update         ;
            mov.b   #0,LCD_update_flag      ;            
mainloop                                    ;
            tst.b   LCD_update_flag         ;
            jnz     mainLCD                 ;
            
            bit.b   #B2                     ;MINUTES++
            jnz     B2_nopressed            ;
            bic.b   #WDTIE,&IE1             ;disable WDT            
            dadd.w  #01h,HOURMIN            ;
            cmp.b   #60h,HOURMIN            ;
            jne     chk1_exit               ;
            and.w   #0FF00h,HOURMIN         ;
chk1_exit   mov.w   #B2S,SPTR               ;B2 string
            call    #TX_String              ;
            call    #LCD_RTC_update         ;
            mov.b   #0,LCD_update_flag      ;            
            bic.b   #LED                    ;
            mov.w   #F2_0,FREQ0             ;
            mov.w   #F2_1,FREQ1             ;
            call    #Beep                   ;
            bis.b   #LED                    ;
B2_nopressed
            bit.b   #B1                     ;HOURS++
            jnz     check_3                 ;
            bic.b   #WDTIE,&IE1             ;disable WDT
            dadd.w  #00100h,HOURMIN         ;
            cmp.w   #01300h,HOURMIN         ;
            jlo     chk2_exit               ;
            and.w   #000FFh,HOURMIN         ;
            bis.w   #00100h,HOURMIN         ;
chk2_exit   mov.w   #B1S,SPTR               ;B1 string
            call    #TX_String              ;
            call    #LCD_RTC_update         ;
            mov.b   #0,LCD_update_flag      ;            
            bic.b   #LED                    ;
            mov.w   #F1_0,FREQ0             ;
            mov.w   #F1_1,FREQ1             ;
            call    #Beep                   ;
            bis.b   #LED                    ;
check_3
            bit.b   #B3                     ;
            jnz     check_4                 ;
            mov.w   #B3S,SPTR               ;B2 string
            call    #TX_String              ;
            call    #LCD_update             ;
            bic.b   #LED                    ;
            mov.w   #F3_0,FREQ0             ;
            mov.w   #F3_1,FREQ1             ;
            call    #Beep                   ;
            bis.b   #LED                    ;
check_4
            bit.b   #B4                     ;
            jnz     check_exit              ;
            mov.w   #B4S,SPTR               ;B2 string
            call    #TX_String              ;
            call    #LCD_update             ;
            bic.b   #LED                    ;
            mov.w   #F4_0,FREQ0             ;
            mov.w   #F4_1,FREQ1             ;
            call    #Beep                   ;
            bis.b   #LED                    ;
check_exit
            jmp     mainloop                ;

ERRVEC                                      ;
            jmp     ERRVEC                  ;
            reti                            ;
;--------------------------------------------
Beep        mov.w   #OUTMOD_4,CCTL0         ;toggle output TA0
            mov.w   #OUTMOD_4,CCTL1         ;toggle output TA1
            mov.w   FREQ0,CCR0              ;frequency period TA0
            mov.w   FREQ1,CCR1              ;frequency period TA1
            bis.w   #MC0+MC1,&TACTL         ;Start Timer_A in up/down
            push    INDEX                   ;
            mov.w   #0,INDEX                ;
Beep_wait   dec.w   INDEX                   ;
            jnz     Beep_wait               ;
            pop     INDEX                   ;
            bic.w   #MC0+MC1,&TACTL         ;Stop Timer_A
            ret                             ;
;--------------------------------------------
HelloS      db      CR,LF,"MSP430-449STK, www.olimex.com/dev"
B1S         db      CR,LF,"B1 pressed!"
B2S         db      CR,LF,"B2 pressed!"
B3S         db      CR,LF,"B3 pressed!"
B4S         db      CR,LF,"B4 pressed!"
;--------------------------------------------
; Interrupt Vectors MSP430F1121             
;--------------------------------------------
            rseg    INTVEC
            dw      ERRVEC                   ;0 reserved
            dw      ERRVEC                   ;1 reserved
            dw      ERRVEC                   ;2 port 1 Vector
            dw      ERRVEC                   ;3 port 2 Vector
            dw      ERRVEC                   ;4 reserved
            dw      ERRVEC                   ;5 reserved
            dw      ERRVEC                   ;6 TIMER_A CC0 Vector
            dw      ERRVEC                   ;7 reserved
            dw      ERRVEC                   ;8 TIMER_A CC1-2 TA Vector
            dw      ERRVEC                   ;9 reserved
            dw      WDT_ISR                  ;10 WDT Vector
            dw      ERRVEC                   ;11 Comparator Vector
            dw      ERRVEC                   ;12 reserved            
            dw      ERRVEC                   ;13 reserved
            dw      ERRVEC                   ;14 NMI Vector
            dw      RESET                   ;15 RESET Vector            
            END