;-------------------------------------------
; UART SUBROUTINES FOR 449 STK
;
; LAST MODIFIED: 04.12.2002
;-------------------------------------------
;
;  RX_Byte     - wait to receive with timeout
;  TX_String   - transmits string terminated with 0
;  TX_Byte     - transmit byte as character
;  TX_Byte_HEX - transmit byte as hex
;  UART1_setup - setups UART1 to 9600,1,8,N
;
;--------------------------------------------
; UART1 routines                    
;--------------------------------------------
UART1_setup
           mov.b    #SSEL1,U1TCTL           ;UCLK =SMCLK
           mov.b    #06Dh,U1BR0             ;Baud rate = 9600 (32x ACLK)
           mov.b    #0,U1BR1                ;
           mov.b    #0,U1MCTL               ;no modulation
           mov.b    #CHAR,U1CTL             ;
           bis.b    #UTXE1+URXE1,ME2        ;enable UART1 Tx,Rx
           bis.b    #03h,P4SEL              ;select UART1 pins
           bis.b    #01h,P4DIR              ;port direction
           ret                              ;
;--------------------------------------------
RX_Byte     push    INDEX                   ;wait for byte reception
            mov.w   #0,INDEX                ;put result in RXTXData
	    mov.b   #0,RXTXData             ;
RX_LOOP     bit.b   #URXIFG1,IFG2           ;wait URXIFG=1
            jnz     RX_done                 ;
            dec.w   INDEX                   ;
            jnz     RX_LOOP                 ;
            ret                             ;
RX_done     mov.b   U1RXBUF,RXTXData        ;
            ret                             ;
;--------------------------------------------
TX_String                                   ;Send ASCII strings terminated with 0
TX_S1       mov.b   @SPTR+,RXTXData         ;SPTR used as pointer
            call    #TX_Byte                ;
            tst.b   0(SPTR)                 ;
            jnz    TX_S1                    ;
            ret                             ;
;--------------------------------------------
TX_Byte_ASCII                               ;Convert and TX byte from DataConv 
            push    DataConv                ;in two ASCII bytes
            call    #NUM_ASCIR              ;
            push    DataConv                ;
            call    #NUM_ASCIA              ;
            ret                             ;
NUM_ASCIR   rrc     2(SP)                   ;
            rrc     2(SP)                   ;
            rrc     2(SP)                   ;
            rrc     2(SP)                   ;
NUM_ASCIA   and     #0fh,2(SP)              ;
            add     #030h,2(SP)             ;
            cmp     #03ah,2(SP)             ;
            jlo     NUM_End                 ;
            add     #039,2(SP)              ;
NUM_End     mov     2(SP),RXTXData          ;
            mov     @SP+,0(SP)              ;
;--------------------------------------------
TX_Byte     bit.b  #UTXIFG1,IFG2            ;Transmits one byte 
            jz     TX_Byte                  ;
            mov.b  RXTXData,U1TXBUF         ;
            ret                             ;
